/* rsmcl.c */
/* Resource Standard Metrics Code Listing */
/* (C) 1996 Robert Mudge
   All Rights Reserved
   M Squared Technologies
   (407) 880-2627
   Email: m2tech@gslink.net
   Use may use this code and program as long as you
   don't use it for commercial use, resell it, publish it
   or change it.  Please contact me for commercial rights to
   use this source.  The use of this program for business
   purposes requires a $25.00 per node license.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <time.h>

#ifndef MAXPATHLEN
#define MAXPATHLEN 255
#endif

#define VERSION "1.1"
#define BUFFERSIZE MAXPATHLEN
#define HEADERSIZE MAXPATHLEN
#define NUMBUFSIZE 41
#define LM_LENGTH 0
#define TM_LENGTH 0
#define PG_LENGTH 60
#define LN_LENGTH 80
#define TOPBAR "------------------------------------------------"
#define BOTBAR "------------------------------------------------"
#define OFFSET 5
#define CONTINUE "     : "
#define PAGEOFFSET 4

/* Global parameters to keep things FAST! */
char TopLine[BUFFERSIZE];
char BotLine[BUFFERSIZE];
char header [HEADERSIZE];
char headerline [BUFFERSIZE];
char leftmargin [NUMBUFSIZE];

FILE * OutPutFile = NULL;
FILE * InPutFile = NULL;

int  ln_length = 0;
int  pg_length = 0;
int  lm_length = 0;
int  tm_length = 0;
time_t t;

void
SetBars ( void )
{
  int i, len;
  strcpy ( TopLine, leftmargin );
  strcat ( TopLine, TOPBAR);
  strcpy ( BotLine, leftmargin );
  strcat ( BotLine, BOTBAR);
  len = strlen(TopLine)-lm_length;
  for ( i=0; i < (ln_length-len); i++ )
  {
     strcat ( TopLine, "-" );
     strcat ( BotLine, "-");
  }
  strcat (TopLine, "\n");
  strcat (BotLine, "\n");
}

char *
GetFilename (char * output, char * input )
{
  int i = 0;
  while ( input[i] != '.' && input[i] )
  {
    output[i] = input[i];
    i++;
  }
  strcat ( output, ".rsm" );
  return ( output );
}

void
PutTopMargin ( void )
{
  int i;
  for ( i=0; i < tm_length; i++ )
  {
    fputs ( "\n", OutPutFile );
  }
}

int
PutLine ( char * input, long line_number )
{
  int i = 0;                   /* counter */
  int j = 0;                   /* counter into input line */
  int numlen = 0;              /* number length */
  int linelen = strlen(input); /* line length */
  int numlines = 0;            /* number of formatted lines */
  char outline [BUFFERSIZE];   /* output line to file */
  char templine [BUFFERSIZE];   /* output line to file */
  char numbuf [NUMBUFSIZE];    /* buffer for the number string */
  char numbuf2 [NUMBUFSIZE];   /* temp num buffer */

  memset( outline, 0, sizeof( outline ) );
  memset( templine, 0, sizeof( templine ) );
  memset( numbuf, 0, sizeof( numbuf ) );
  memset( numbuf2, 0, sizeof( numbuf2 ) );

  /* setup number string */
  sprintf(numbuf2,"%ld",line_number);
  numlen = strlen(numbuf2);
  numbuf[0] = '\0';

  /* pad the number string with spaces to the offset */
  for ( i=0; i < (OFFSET - numlen ); i++ )
  {
    strcat ( numbuf, " " );
  }

  strcat ( numbuf, numbuf2 );
  strcat ( numbuf, ": \0" );

  /* format the outgoing line */
  if ( linelen > (ln_length-OFFSET-2) )
  {

    numlines = ceil ( ((float)linelen/((float)(ln_length-OFFSET-2))) );
    strncpy ( templine, input, (ln_length-OFFSET-2) );
    strcat ( templine, "\0" );
    sprintf( outline,"%s%s%s", leftmargin, numbuf, templine );

    strcat ( outline, "\n" );
    j = ln_length-OFFSET-2;

    while ( j < linelen )
    {
      strcat ( outline, leftmargin );
      strcat ( outline, CONTINUE );
      memset( templine, 0, sizeof( templine ) );
      for ( i=0; i < (ln_length-OFFSET-2); i++ )
      {
        templine[i] = input[j];
        j++;
      }
      strcat ( templine, "\n" );
      strcat ( outline, templine );
    }
    outline[strlen(outline)-1] = '\0';
  }
  else
  {
    sprintf( outline,"%s%s%s", leftmargin, numbuf, input );
    numlines = 1;
  }

  fputs ( outline, OutPutFile);
  return ( numlines );
}

void
SetLeftMargin ( void )
{
  int i;
  for ( i=0; i<lm_length; i++ )
  {
    strcat(leftmargin, " ");
  }
}

void
SetHeaderLine( char * filename, int pagenum )
{
  int len = 0;
  int i = 0;
  char numbuf[41];
  sprintf ( headerline, "%s%s%s", leftmargin,
            "File: ", filename );
  len = strlen ( header );
  for ( i=0; i < ( (0.5 * ln_length)
                 - ( (0.5 * len) + (0.5 * strlen(headerline))) ); i++ )
  {
    strcat(headerline, " " );
  }
  strcat(headerline, header );
  len = strlen ( headerline );
  sprintf ( numbuf,"%d",pagenum );
  for ( i=0; i< (ln_length- (len+strlen(numbuf)+6-lm_length )); i++ )
  {
    strcat ( headerline, " " );
  }
  strcat ( headerline, "Page: " );
  strcat ( headerline, numbuf );
  strcat( headerline, "\n" );
  fputs ( headerline, OutPutFile );
  sprintf ( headerline, "%s%s%s", leftmargin, "Print Date: ", ctime(&t) );
  fputs ( headerline, OutPutFile );
  fputs ( TopLine, OutPutFile );
}

int
main ( int argc, char ** argv )
{

  long int  linecount  = 1;
  int  pagecount  = 1;
  long int  localline = 1;

  char filename [MAXPATHLEN];
  char outputfile [MAXPATHLEN];
  char linebuffer [MAXPATHLEN];

  memset ( header,     0, sizeof( header ) );
  memset ( leftmargin, 0, sizeof( leftmargin ) );
  memset ( filename,   0, sizeof( filename) );
  memset ( outputfile, 0, sizeof( outputfile ) );
  memset ( linebuffer, 0, sizeof( linebuffer ) );

  t = time(NULL);

  /* show syntax for no parameters */
  if ( argc == 1 ||
       ( argc == 2 && argv[1][0] == '-' && argv[1][1] == 'h') )
  {
    printf ( "Resource Standard Metrics Code Listing\n"
             "Version %s\n"
             "(C) 1996 Robert Mudge\n"
             "Syntax: %s\n"
             "        filename <line length> <page length> <left margin> <top margin>\n"
             "        <header string>\n\n",
             VERSION, argv[0] );
    printf ( "        The input source code is processed by adding a\n"
             "        line number to each line in the source file.\n"
             "        Program Defaults:  line length: %d columns  page length: %d lines\n"
             "                           left margin: %d columns  top margin : %d lines\n"
             "                           header     : \"NONE\", Use Quotes \n\n",
             LN_LENGTH, PG_LENGTH, LM_LENGTH, TM_LENGTH );
  printf (
  "rsmcl will create a file formatted for printing, terminated in .rsm\n"
  "Typical use: rsmlc filename.c 72 55 2 1 \"File Header\"\n"
  "A file may be printed by <DOS>  copy filename.rsm prn\n"
  "                         <UNIX> lpr filename.rsm\n\n"
  "The use of this program is FREE for NONCOMMERCIAL purposes.  It may not\n"
  "be changed or resold.  Please contact the author for commcercial licenses.\n"
  "Commercial use is $25.00 per node or $500 per site licenses.  Site license\n"
  "will provide a right to modify the source code as a site desires but the\n"
  "code may not be resold.  (C) 1996 Robert Mudge, M Squared Technologies\n"
  "       Contact: Robert Mudge, m2tech@gslink.net, (407) 880-2627"
  );

    return 0;
  }

  if ( argc >= 2 )
  {
    strcpy ( filename, argv[1] );
  }

  if ( argc >= 3 )
  {
    ln_length = atoi(argv[2]);
  }
  else
  {
    ln_length = LN_LENGTH;
  }

  if ( argc >= 4 )
  {
    pg_length = atoi( argv[3] );
  }
  else
  {
    pg_length = PG_LENGTH;
  }

  if ( argc >= 5 )
  {
    lm_length = atoi( argv[4] );
  }
  else
  {
    lm_length = LM_LENGTH;
  }

  if ( argc >= 6 )
  {
    tm_length = atoi( argv[5] );
  }
  else
  {
    tm_length = TM_LENGTH;
  }

  if ( argc >= 7 )
  {
    strcpy ( header, argv[6] );
  }
  else
  {
    header[0] = '\0';
  }

  /* check opening of the data file */
  InPutFile = fopen ( filename, "r" );

  if ( !InPutFile )
  {
    fprintf ( stderr, "Cannot open the specified file: %s", filename );
    return (-1);
  }

  GetFilename(outputfile, filename);

  OutPutFile = fopen ( outputfile, "w" );
  if ( !OutPutFile )
  {
    fprintf ( stderr, "Cannot open the specified file: %s", outputfile );
    return (-1);
  }

  printf ("\nResource Standard Metrics List Code\n"
          "(C) 1996 Robert Mudge\n"
          "Processing...\n"
          "From File: %s\n"
          "  To File: %s\n"
          "Operating Parameters:  Line Length: %d\tPage Length: %d\n"
          "                       Left Margin: %d\tTop Margin : %d\n\n",
          filename,outputfile,ln_length,pg_length,
          lm_length, tm_length );

  SetLeftMargin();
  SetBars();
  PutTopMargin();
  SetHeaderLine( filename, linecount );

  for (;;)
  {
    fgets (linebuffer, sizeof(linebuffer), InPutFile);
    if ( feof( InPutFile ) )
    {
     break;
    }

    localline += PutLine ( linebuffer, linecount );

    if ( localline > (pg_length-PAGEOFFSET) )
    {
      fputs ( BotLine, OutPutFile );
      fputs ( "\n", OutPutFile );
      pagecount++;
      SetHeaderLine ( filename, pagecount );
      localline = 1;
    }
    linecount++;
  }

  while ( localline <= (pg_length-PAGEOFFSET) )
  {
    sprintf(linebuffer,"%s","\n" );
    fputs( linebuffer, OutPutFile );
    localline++;
  }

  fputs ( BotLine, OutPutFile );

  printf ( "Processed %ld Lines\n", linecount-1 );

  fclose ( InPutFile );
  fclose ( OutPutFile );
  return (1);
}
